/*
 * Copyright (c) 2003 FlashCode <flashcode@flashtux.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __GMEMO_H
#define __GMEMO_H   1

#include <gtk/gtk.h>


// debug mode, 0 for normal use, 1 or 2 for developers (2 = full debug messages)
#define DEBUG               0

// internationalization
#define _(String) gettext (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

// gmemo messages, warnings, errors

#define GMEMO_DEBUG(fmt, argz...) \
	gmemo_message("[DEBUG] gmemo: " fmt, ##argz)

#define GMEMO_MESSAGE(fmt, argz...) \
	gmemo_message("gmemo: " fmt, ##argz)

#define GMEMO_WARNING(fmt, argz...) \
	gmemo_message("gmemo warning: " fmt, ##argz)

#define GMEMO_ERROR(fmt, argz...) \
	gmemo_message("gmemo error: " fmt, ##argz)


#define TYPE_GTK            0
#define TYPE_WMAKER         1
#define TYPE_LAST           TYPE_WMAKER
#define GMEMO_NUM_TYPES     (TYPE_LAST + 1)

#define COLOR_BG_WINDOW     0
#define COLOR_FG_DIGITS     1
#define COLOR_FG_DATE       2
#define COLOR_FG_LINE1      3
#define COLOR_FG_LINE2      4
#define COLOR_BG_ALERT      5
#define COLOR_LAST          COLOR_BG_ALERT
#define GMEMO_NUM_COLORS    (COLOR_LAST + 1)

#define FONT_DATE           0
#define FONT_LINE1          1
#define FONT_LINE2          2
#define FONT_LAST           FONT_LINE2
#define GMEMO_NUM_FONTS     (FONT_LAST + 1)


struct t_date_time
{
    GDate   date;
    guint   hour : 5;
    guint   min  : 6;
    guint   sec  : 6;
};

extern int                  gmemo_type;
extern gboolean             gmemo_daemon;
extern gboolean             gmemo_quiet;
extern gboolean             gmemo_verbose;
extern gchar                gmemo_display[256];
extern gint                 gmemo_win_width;
extern gint                 gmemo_win_height;
extern gint                 gmemo_size_digits;
extern gboolean             gmemo_display_seconds;
extern gchar                gmemo_colors[GMEMO_NUM_COLORS][64];
extern gchar                gmemo_colors_default_gtk[GMEMO_NUM_COLORS][64];
extern gchar                gmemo_colors_default_wmaker[GMEMO_NUM_COLORS][64];
extern gchar                gmemo_fonts[GMEMO_NUM_FONTS][128];
extern gchar                gmemo_fonts_default[GMEMO_NUM_FONTS][128];

extern gchar                gmemo_tab_types[GMEMO_NUM_TYPES][64];

extern struct t_date_time   date_of_day;

extern gchar                new_time[16];
extern gchar                old_time[16];

extern gint                 last_min;
extern gint                 last_day;

extern gint                 mouse_over_config;

extern gint                 chiffre[10];

extern gint                 coord_barre[7][2];
extern gint                 long_barre[7][3];

extern gchar                day_name[7][9];
extern gchar                short_day_name[7][4];
extern gchar                month_name[12][10];
extern gchar                short_month_name[12][5];
extern gchar                feast_type[12][31];
extern gchar                feast[12][31][128];

extern gint                 inverse;


extern void gmemo_display_usage();
extern void gmemo_display_licence();
extern void gmemo_message(gchar *, ...);
extern void signal_hup(int);
extern void display_time(gboolean);
extern gboolean update_time(gboolean);

#endif /* gmemo.h */
