/*
 * Copyright (c) 2003 FlashCode <flashcode@flashtux.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <gtk/gtk.h>
#ifdef WIN32
    #include <io.h>
    #include <libxml/tree.h>
    #include <libxml/parser.h>
#else
    #include <sys/stat.h>
    #include <gnome-xml/tree.h>
    #include <gnome-xml/parser.h>
#endif

#include "gmemo.h"
#include "gmemo_loadsave.h"
#include "gmemo_memo.h"


#ifdef WIN32
    #define XMLROOT     children
#else
    #define XMLROOT     root
#endif


/*
 * load_feasts : read gmemo feasts (xml format)
 */

gboolean load_feasts()
{
    gchar       *home_dir;
    gchar       gmemo_xmlfile[256];
    xmlDocPtr   doc;
    xmlNodePtr  node;
    xmlChar     *date, *type, *name;
    gint        day, month;
    
    if (gmemo_verbose)
        GMEMO_MESSAGE("reading feasts");
    
    home_dir = (gchar *)g_get_home_dir();
    if (home_dir == NULL)
        strcpy(gmemo_xmlfile, GMEMO_FEASTS_FILE);
    else
        sprintf(gmemo_xmlfile, "%s/.gmemo/" GMEMO_FEASTS_FILE, home_dir);
    if (g_file_test(gmemo_xmlfile, G_FILE_TEST_EXISTS) == FALSE)
    {
        GMEMO_WARNING("feasts file not found (%s)... no feast will be displayed !", 
            gmemo_xmlfile);
        return FALSE;
    }
    
    doc = xmlParseFile(gmemo_xmlfile);
    if (doc == NULL)
    {
        GMEMO_WARNING("error parsing feasts file... using default config");
        return FALSE;
    }
    gdk_beep();
    node = xmlDocGetRootElement(doc);
    if (node == NULL)
    {
        GMEMO_WARNING("empty feasts file... using default config");
        return FALSE;
    }
    if (xmlStrcmp(node->name, (const xmlChar *)"feasts"))
    {
        GMEMO_WARNING("wrong type for xml configuration... using default config");
        return FALSE;
    }
    node = node->xmlChildrenNode;
    while (node)
    {
        if (!xmlStrcmp(node->name, (const xmlChar *)"feast"))
        {
            date = xmlGetProp(node, "date");
            type = xmlGetProp(node, "type");
            name = xmlGetProp(node, "name");
            #if DEBUG >= 2
            GMEMO_DEBUG("item read: date=%s type=%s name=%s", date, type, name);
            #endif
            sscanf(date, "%d/%d", &month, &day);
            feast_type[month-1][day-1] = type[0];
            strcpy(feast[month-1][day-1], name);
        }
        node = node->next;
    }
    if (gmemo_verbose)
        GMEMO_MESSAGE("feasts loaded");
    return TRUE;
}

/*
 * read_memos : read memos
 */

gboolean read_memos()
{
    gchar           *home_dir;
    gchar           gmemo_xmlfile[256];
    xmlDocPtr       doc;
    xmlNodePtr      node, node_memo;
    xmlChar         *short_lib, *long_lib, *alarm_date, *alarm_time, *last_alarm_date, *last_alarm_time;
    xmlChar         *repeat_type, *interval, *repeat_count, *sound_file, *cmd_exec;
    gint            year, month, day, hour, min;
    t_memo          *memo;
    gint            i;
    
    if (gmemo_verbose)
        GMEMO_MESSAGE("reading memos file");
    
    home_dir = (gchar *)g_get_home_dir();
    if (home_dir == NULL)
        strcpy(gmemo_xmlfile, GMEMO_MEMOS_FILE);
    else
        sprintf(gmemo_xmlfile, "%s/.gmemo/" GMEMO_MEMOS_FILE, (home_dir == NULL) ? "." : home_dir);
    if (g_file_test(gmemo_xmlfile, G_FILE_TEST_EXISTS) == FALSE)
    {
        GMEMO_MESSAGE("xml configuration file not found (%s)... starting with empty config", 
            gmemo_xmlfile);
        return FALSE;
    }
    
    doc = xmlParseFile(gmemo_xmlfile);
    if (doc == NULL)
    {
        GMEMO_WARNING("error parsing xml file... no memo loaded");
        return FALSE;
    }
    node = xmlDocGetRootElement(doc);
    if (node == NULL)
    {
        GMEMO_WARNING("empty xml file... no memo loaded");
        return FALSE;
    }
    if (xmlStrcmp(node->name, (const xmlChar *)"gmemo"))
    {
        GMEMO_WARNING("wrong xml file... no memo loaded");
        return FALSE;
    }
    node = node->xmlChildrenNode;
    while (node != NULL)
    {
        if (!xmlStrcmp(node->name, (const xmlChar *)"memo"))
        {
            short_lib = NULL;
            long_lib = NULL;
            alarm_date = NULL;
            alarm_time = NULL;
            last_alarm_date = NULL;
            last_alarm_time = NULL;
            repeat_type = NULL;
            interval = NULL;
            repeat_count = NULL;
            sound_file = NULL;
            cmd_exec = NULL;
            
            node_memo = node->xmlChildrenNode;
            while (node_memo != NULL)
            {
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"lib"))
                    short_lib = xmlNodeListGetString(doc, node_memo->xmlChildrenNode, 1);
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"longlib"))
                    long_lib = xmlNodeListGetString(doc, node_memo->xmlChildrenNode, 1);
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"alarm"))
                {
                    alarm_date = xmlGetProp(node_memo, "date");
                    alarm_time = xmlGetProp(node_memo, "time");
                }
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"last_alarm"))
                {
                    last_alarm_date = xmlGetProp(node_memo, "date");
                    last_alarm_time = xmlGetProp(node_memo, "time");
                }
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"repeat"))
                {
                    repeat_type = xmlGetProp(node_memo, "unit");
                    interval = xmlGetProp(node_memo, "interval");
                    repeat_count = xmlGetProp(node_memo, "count");
                }
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"sound"))
                {
                    sound_file = xmlGetProp(node_memo, "file");
                }
                if (!xmlStrcmp(node_memo->name, (const xmlChar *)"exec"))
                {
                    cmd_exec = xmlGetProp(node_memo, "cmd");
                }
                node_memo = node_memo->next;
            }
            
            if ( ( (short_lib == NULL) && (long_lib == NULL) ) || (alarm_date == NULL) )
                GMEMO_WARNING("invalid memo from gmemo file, memo not loaded");
            else
            {
                #if DEBUG >= 1
                //GMEMO_DEBUG("memo read: %s  %s %s  %s  %s %s %s", short_lib, long_lib, alarm, time, repeat_type, interval, repeat_count);
                #endif
                
                memo = (t_memo *)malloc(sizeof(t_memo));
                if (short_lib == NULL)
                    memo->short_lib = NULL;
                else
                {
                    memo->short_lib = (gchar *)g_malloc(strlen(short_lib)+1);
                    strcpy(memo->short_lib, short_lib);
                }
                if (long_lib == NULL)
                    memo->long_lib = NULL;
                else
                {
                    memo->long_lib = (gchar *)g_malloc(strlen(long_lib)+1);
                    strcpy(memo->long_lib, long_lib);
                }
                sscanf(alarm_date, "%d/%d/%d", &year, &month, &day);
                if (alarm_time != NULL)
                    sscanf(alarm_time, "%d:%d", &hour, &min);
                else
                {
                    hour = 0;
                    min = 0;
                }
                memo->alarm = date_to_glong(year, month, day, hour, min, 0);
                if (last_alarm_date == NULL)
                    memo->last_alarm = 0;
                else
                {
                    sscanf(last_alarm_date, "%d/%d/%d", &year, &month, &day);
                    if (last_alarm_time != NULL)
                        sscanf(last_alarm_time, "%d:%d", &hour, &min);
                    else
                    {
                        hour = 0;
                        min = 0;
                    }
                    memo->last_alarm = date_to_glong(year, month, day, hour, min, 0);
                }
                memo->repeat_type = REPEAT_NONE;
                if (repeat_type != NULL)
                {
                    memo->repeat_type = -1;
                    for (i = 0; i < NUM_REPEAT; i++)
                        if (strcasecmp(repeat_type, repeat_strings[i]) == 0)
                            memo->repeat_type = i;
                    if (memo->repeat_type == -1)
                    {
                        GMEMO_WARNING("unknown repeat unit time, repeat discarded for memo '%s'", memo->short_lib);
                        memo->repeat_type = REPEAT_NONE;
                    }
                }
                if (memo->repeat_type == REPEAT_NONE)
                {
                    memo->interval = 0;
                    memo->repeat_remaining = 0;
                }
                else
                {
                    if (interval == NULL)
                    {
                        GMEMO_WARNING("no interval specified for memo '%s', repeat discarded", memo->short_lib);
                        memo->repeat_type = REPEAT_NONE;
                        memo->interval = 0;
                        memo->repeat_remaining = 0;
                    }
                    else
                    {
                        memo->interval = (gint)(atoi(interval));
                        if (repeat_count == NULL)
                            memo->repeat_remaining = -1;    // infinite repeats if no count specified
                        else
                            memo->repeat_remaining = (gint)(atoi(repeat_count));
                    }
                }
                if (sound_file == NULL)
                    memo->sound_file = NULL;
                else
                {
                    memo->sound_file = (gchar *)g_malloc(strlen(sound_file)+1);
                    strcpy(memo->sound_file, sound_file);
                }
                if (cmd_exec == NULL)
                {
                    memo->cmd_exec = NULL;
                }
                else
                {
                    memo->cmd_exec = (gchar *)g_malloc(strlen(cmd_exec)+1);
                    strcpy(memo->cmd_exec, cmd_exec);
                }
                if ( ( (memo->last_alarm == 0) && (memo->repeat_type == REPEAT_NONE) )
                    || ( (memo->repeat_type != REPEAT_NONE) && (memo->repeat_remaining == 0) ) )
                    memo->active = FALSE;
                else
                    memo->active = TRUE;
                set_first_alarm(memo);
                memos = g_slist_append(memos, memo);
            }
        }
        node = node->next;
    }
    if (gmemo_verbose)
        GMEMO_MESSAGE("loading memos... done");
    #if DEBUG >= 1
    display_memos(memos);
    #endif
    return TRUE;
}

/*
 * write_memos : writes memos
 */

gboolean write_memos()
{
    gchar       *home_dir;
    gchar       gmemo_dir[256], gmemo_xmlfile[256];
    xmlDocPtr   doc;
    xmlNodePtr  node, node_memo, newelement, repeat;
    xmlAttrPtr  newattr;
    GSList      *ptr_memos;
    t_memo      *memo;
    struct tm   *ptr_tm;
    gchar       string[256];
    
    if (gmemo_verbose)
        GMEMO_MESSAGE("writing memos file");
    
    home_dir = (gchar *)g_get_home_dir();
    if (home_dir == NULL)
        strcpy(gmemo_dir, ".");
    else
    {
        sprintf(gmemo_dir, "%s/.gmemo", home_dir);
        if (g_file_test(gmemo_dir, G_FILE_TEST_IS_DIR) == FALSE)
        {
            GMEMO_MESSAGE("directory %s doesn't exist... creating it...", gmemo_dir);
            #ifdef WIN32
            mkdir(gmemo_dir);
            #else
            mkdir(gmemo_dir, 0x777);
            #endif
        }
    }
    
    sprintf(gmemo_xmlfile, "%s/" GMEMO_MEMOS_FILE, gmemo_dir);
    
    doc = xmlNewDoc("1.0");
    doc->XMLROOT = xmlNewDocNode(doc, NULL, "gmemo", NULL);

    node = xmlDocGetRootElement(doc);
    
    ptr_memos = memos;
    while (ptr_memos != NULL)
    {
        memo = (t_memo *)(ptr_memos->data);
        node_memo = xmlNewTextChild(node, NULL, "memo", NULL);
        newelement = xmlNewTextChild(node_memo, NULL, "lib", memo->short_lib);
        newelement = xmlNewTextChild(node_memo, NULL, "longlib", memo->long_lib);
        ptr_tm = localtime(&(memo->alarm));
        sprintf(string, "%04d/%02d/%02d %02d:%02d",
            ptr_tm->tm_year+1900, ptr_tm->tm_mon+1, ptr_tm->tm_mday,
            ptr_tm->tm_hour, ptr_tm->tm_min);
        newelement = xmlNewTextChild(node_memo, NULL, "alarm", string);
        if (memo->last_alarm > 0)
        {
            ptr_tm = localtime(&(memo->last_alarm));
            sprintf(string, "%04d/%02d/%02d %02d:%02d",
                ptr_tm->tm_year+1900, ptr_tm->tm_mon+1, ptr_tm->tm_mday,
                ptr_tm->tm_hour, ptr_tm->tm_min);
            newelement = xmlNewTextChild(node_memo, NULL, "last_alarm", string);
        }
        if (memo->repeat_type != REPEAT_NONE)
        {
            repeat = xmlNewTextChild(node_memo, NULL, "repeat", NULL);
            newattr = xmlNewProp(repeat, "unit", repeat_strings[memo->repeat_type]);
            sprintf(string, "%d", memo->interval);
            newattr = xmlNewProp(repeat, "interval", string);
            if (memo->repeat_remaining >= 0)
            {
                sprintf(string, "%d", memo->repeat_remaining);
                newattr = xmlNewProp(repeat, "count", string);
            }
        }
        ptr_memos = ptr_memos->next;
    }
    
    GMEMO_DEBUG("xmlsaveformatfile return code: %d", 
        xmlSaveFile(gmemo_xmlfile, doc));
    return TRUE;
}
